﻿########################################################################################################
# Scriptname: ExportCert.ps1
# Author: Michael Rüefli
# Purpose: Export certificates local certificate store (Machine or User) to a PKCS12 file format
# Version: 1.0.1
# 
# 
# Fixed Issues / Changes:
# V 1.0.1 / Fixed Export where no filter has been specified. Changed the autogenerated password strenght
########################################################################################################

function ConvertToSid([STRING]$NtAccount) 
{
	$result = (New-Object system.security.principal.NtAccount($NTaccount)).translate([system.security.principal.securityidentifier])
	return $result.value
}

#Get the Arguments
$exportpath = $args[0]
$certstore = $args[1]
$issuer_filter = $args[2]

#Check the Args
If ($args.count -lt 2)
{
	Write-host "Too less arguments! Usage: ExportCert.ps1 <exportpath> <certstore> [<filter> optional>" -ForegroundColor red
	write-host "Example: Powershell.exe ExportCert.ps1 H:\Certs CurrentUser DC=LOC" -ForegroundColor blue
	exit
}

#Error Handler
Trap [Exception]{continue}

#Check Exportpath, if not there create it
If ((Test-Path -Path $exportpath) -ne $True)
{
	New-Item -Path $exportpath -ItemType Directory 
}

#Get certificates in store
If ($issuer_filter)
{
	$HKCUCerts = (dir cert:\$certstore\My | ? { $_.Issuer -notmatch $issuer_filter})
}
Else
{
	$HKCUCerts = (dir cert:\$certstore\My)
}

#process each certificate
Foreach ($cert in $HKCUCerts)
{
	$friendlyname = $cert.FriendlyName
	$type = [System.Security.Cryptography.X509Certificates.X509ContentType]::pfx
	$username = $env:USERNAME
	$sid = ConvertToSid $username
	$pass = "$friendlyname-$sid"
	$pass_secure = ConvertTo-SecureString -AsPlainText $pass -Force
	$bytes = $cert.export($type, $pass)
	[System.IO.File]::WriteAllBytes("$exportpath\$friendlyname.pfx", $bytes)
}



